@@ -27,7 +27,6 @@ option('header', 1, 'parse first N rows as column names', replay=True)
 option('load_lazy', False, 'load subsheets always (False) or lazily (True)')
 
 theme('force_256_colors', False, 'use 256 colors even if curses reports fewer')
-theme('use_default_colors', True, 'curses use default terminal colors')
 
 theme('disp_note_none', '⌀',  'visible contents of a cell whose value is None')
 theme('disp_truncator', '…', 'indicator that the contents are only partially visible')
@@ -62,7 +61,7 @@ theme('disp_endbot_sep', '║', '') # ╽╿┃╜‖
 theme('disp_selected_note', '•', '') #
 theme('disp_sort_asc', '↑↟⇞⇡⇧⇑', 'characters for ascending sort') # ↑▲↟↥↾↿⇞⇡⇧⇈⤉⤒⥔⥘⥜⥠⍏˄ˆ
 theme('disp_sort_desc', '↓↡⇟⇣⇩⇓', 'characters for descending sort') # ↓▼↡↧⇂⇃⇟⇣⇩⇊⤈⤓⥕⥙⥝⥡⍖˅ˇ
-theme('color_default', 'white', 'the default color')
+theme('color_default', 'white on black', 'the default fg and bg colors')
 theme('color_default_hdr', 'bold', 'color of the column headers')
 theme('color_bottom_hdr', 'underline', 'color of the bottom header row')
 theme('color_current_row', 'reverse', 'color of the cursor row')
@@ -76,8 +75,8 @@ option('name_joiner', '_', 'string to join sheet or column names')
 option('value_joiner', ' ', 'string to join display values')
 
 
-def splitcell(s, width=0):
-    if width <= 0 or not options.textwrap_cells:
+def splitcell(sheet, s, width=0):
+    if width <= 0 or not sheet.options.textwrap_cells:
         return [s]
 
     ret = []
@@ -246,7 +245,7 @@ class TableSheet(BaseSheet):
             try:
                 r = colorizer.func(self, col, row, value)
                 if r:
-                    colorstack.append(colorizer.coloropt if colorizer.coloropt else r)
+                    colorstack.append((colorizer.precedence, colorizer.coloropt if colorizer.coloropt else r))
             except Exception as e:
                 vd.exceptionCaught(e)
 
@@ -258,6 +257,8 @@ class TableSheet(BaseSheet):
             self.rows.append(row)
         else:
             self.rows.insert(index, row)
+            if self.cursorRowIndex and self.cursorRowIndex >= index:
+                self.cursorRowIndex += 1
         return row
 
     def newRow(self):
@@ -486,6 +487,10 @@ class TableSheet(BaseSheet):
 
     def addColumn(self, *cols, index=None):
         'Insert all *cols* into columns at *index*, or append to end of columns if *index* is None.  Return first column.'
+        if not cols:
+            vd.warning('no columns to add')
+            return
+
         for i, col in enumerate(cols):
             vd.addUndo(self.columns.remove, col)
             if index is None:
@@ -595,8 +600,8 @@ class TableSheet(BaseSheet):
 
     def calcColLayout(self):
         'Set right-most visible column, based on calculation.'
-        minColWidth = len(options.disp_more_left)+len(options.disp_more_right)+2
-        sepColWidth = len(options.disp_column_sep)
+        minColWidth = len(self.options.disp_more_left)+len(self.options.disp_more_right)+2
+        sepColWidth = len(self.options.disp_column_sep)
         winWidth = self.windowWidth
         self._visibleColLayout = {}
         x = 0
@@ -608,8 +613,8 @@ class TableSheet(BaseSheet):
                 # handle delayed column width-finding
                 col.width = max(col.getMaxWidth(vrows), minColWidth)
                 if vcolidx != self.nVisibleCols-1:  # let last column fill up the max width
-                    col.width = min(col.width, options.default_width)
-            width = col.width if col.width is not None else options.default_width
+                    col.width = min(col.width, self.options.default_width)
+            width = col.width if col.width is not None else self.options.default_width
             if col in self.keyCols:
                 width = max(width, 1)  # keycols must all be visible
             if col in self.keyCols or vcolidx >= self.leftVisibleColIndex:  # visible columns
@@ -632,9 +637,9 @@ class TableSheet(BaseSheet):
         if vcolidx == self.cursorVisibleColIndex:
             hdrcattr = update_attr(hdrcattr, colors.color_current_hdr, 2)
 
-        C = options.disp_column_sep
+        C = self.options.disp_column_sep
         if (self.keyCols and col is self.keyCols[-1]) or vcolidx == self.rightVisibleColIndex:
-            C = options.disp_keycol_sep
+            C = self.options.disp_keycol_sep
 
         x, colwidth = self._visibleColLayout[vcolidx]
 
@@ -657,7 +662,7 @@ class TableSheet(BaseSheet):
                 name += hdrs[::-1][h-i-1]
 
             if len(name) > colwidth-1:
-                name = name[:colwidth-len(options.disp_truncator)] + options.disp_truncator
+                name = name[:colwidth-len(self.options.disp_truncator)] + self.options.disp_truncator
 
             if i == h-1:
                 hdrcattr = update_attr(hdrcattr, colors.color_bottom_hdr, 5)
@@ -672,7 +677,7 @@ class TableSheet(BaseSheet):
 
         try:
             if vcolidx == self.leftVisibleColIndex and col not in self.keyCols and self.nonKeyVisibleCols.index(col) > 0:
-                A = options.disp_more_left
+                A = self.options.disp_more_left
                 scr.addstr(y, x, A, sepcattr.attr)
         except ValueError:  # from .index
             pass
@@ -686,7 +691,7 @@ class TableSheet(BaseSheet):
         vd.clearCaches()
 
         if not self.columns:
-            if options.debug:
+            if self.options.debug:
                 self.addColumn(Column())
             else:
                 return
@@ -694,21 +699,21 @@ class TableSheet(BaseSheet):
         drawparams = {
             'isNull': self.isNullFunc(),
 
-            'topsep': options.disp_rowtop_sep,
-            'midsep': options.disp_rowmid_sep,
-            'botsep': options.disp_rowbot_sep,
-            'endsep': options.disp_rowend_sep,
-            'keytopsep': options.disp_keytop_sep,
-            'keymidsep': options.disp_keymid_sep,
-            'keybotsep': options.disp_keybot_sep,
-            'endtopsep': options.disp_endtop_sep,
-            'endmidsep': options.disp_endmid_sep,
-            'endbotsep': options.disp_endbot_sep,
+            'topsep': self.options.disp_rowtop_sep,
+            'midsep': self.options.disp_rowmid_sep,
+            'botsep': self.options.disp_rowbot_sep,
+            'endsep': self.options.disp_rowend_sep,
+            'keytopsep': self.options.disp_keytop_sep,
+            'keymidsep': self.options.disp_keymid_sep,
+            'keybotsep': self.options.disp_keybot_sep,
+            'endtopsep': self.options.disp_endtop_sep,
+            'endmidsep': self.options.disp_endmid_sep,
+            'endbotsep': self.options.disp_endbot_sep,
 
-            'colsep': options.disp_column_sep,
-            'keysep': options.disp_keycol_sep,
-            'selectednote': options.disp_selected_note,
-            'disp_truncator': options.disp_truncator,
+            'colsep': self.options.disp_column_sep,
+            'keysep': self.options.disp_keycol_sep,
+            'selectednote': self.options.disp_selected_note,
+            'disp_truncator': self.options.disp_truncator,
         }
 
         self._rowLayout = {}  # [rowidx] -> (y, height)
@@ -735,7 +740,7 @@ class TableSheet(BaseSheet):
             y += self.drawRow(scr, row, self.topRowIndex+rowidx, y, rowcattr, maxheight=self.windowHeight-y-1, **drawparams)
 
         if vcolidx+1 < self.nVisibleCols:
-            scr.addstr(headerRow, self.windowWidth-2, options.disp_more_right, colors.color_column_sep)
+            scr.addstr(headerRow, self.windowWidth-2, self.options.disp_more_right, colors.color_column_sep)
 
         scr.refresh()
 
@@ -755,13 +760,13 @@ class TableSheet(BaseSheet):
 
                     try:
                         if isNull and isNull(cellval.value):
-                            cellval.note = options.disp_note_none
+                            cellval.note = self.options.disp_note_none
                             cellval.notecolor = 'color_note_type'
                     except (TypeError, ValueError):
                         pass
 
                     if col.voffset or col.height > 1:
-                        lines = splitcell(cellval.display, width=colwidth-2)
+                        lines = splitcell(self, cellval.display, width=colwidth-2)
                     else:
                         lines = [cellval.display]
                     displines[vcolidx] = (col, cellval, lines)
@@ -883,7 +888,7 @@ class TableSheet(BaseSheet):
             return height
 
 vd.rowNoters = [
-        lambda sheet, row: sheet.isSelected(row) and options.disp_selected_note,
+        lambda sheet, row: sheet.isSelected(row) and sheet.options.disp_selected_note,
 ]
 
 Sheet = TableSheet  # deprecated in 2.0 but still widely used internally
@@ -895,7 +900,7 @@ class SequenceSheet(Sheet):
         self.columns = []
         for i, colnamelines in enumerate(itertools.zip_longest(*headerrows, fillvalue='')):
             colnamelines = ['' if c is None else c for c in colnamelines]
-            self.addColumn(ColumnItem(''.join(colnamelines), i))
+            self.addColumn(ColumnItem(''.join(map(str, colnamelines)), i))
 
         self._rowtype = namedlist('tsvobj', [(c.name or '_') for c in self.columns])
 
@@ -912,7 +917,7 @@ class SequenceSheet(Sheet):
 
     def optlines(self, it, optname):
         'Generate next options.<optname> elements from iterator with exceptions wrapped.'
-        for i in range(options.getobj(optname, self)):
+        for i in range(self.options.getobj(optname, self)):
             try:
                 yield next(it)
             except StopIteration:
@@ -947,7 +952,7 @@ class IndexSheet(Sheet):
 
     columns = [
         ColumnAttr('name'),
-        ColumnAttr('rows', 'nRows', type=int),
+        ColumnAttr('rows', 'nRows', type=int, width=9),
         ColumnAttr('cols', 'nCols', type=int),
         ColumnAttr('keys', 'keyColNames'),
         ColumnAttr('source'),
@@ -965,8 +970,8 @@ class IndexSheet(Sheet):
             if vs.name == k:
                 return vs
 
-    def addRow(self, sheet):
-        super().addRow(sheet)
+    def addRow(self, sheet, **kwargs):
+        super().addRow(sheet, **kwargs)
         if not self.options.load_lazy:
             sheet.ensureLoaded()
 
@@ -976,7 +981,7 @@ class SheetsSheet(IndexSheet):
         ColumnAttr('name'),
         ColumnAttr('type', '__class__.__name__'),
         ColumnAttr('pane', type=int),
-        ColumnAttr('shortcut'),
+        Column('shortcut', getter=lambda c,r: getattr(r, 'shortcut'), setter=lambda c,r,v: setattr(r, '_shortcut', v)),
         ColumnAttr('nRows', type=int),
         ColumnAttr('nCols', type=int),
         ColumnAttr('nVisibleCols', type=int),
@@ -1025,11 +1030,19 @@ def push(vd, vs, pane=0):
     if not isinstance(vs, BaseSheet):
         return  # return instead of raise, some commands need this
 
-    vs.vd = vd
-    vs.pane = pane or vd.activePane
     if vs in vd.sheets:
         vd.sheets.remove(vs)
 
+    vs.vd = vd
+    if pane == -1:
+        vs.pane = 2 if vd.activePane == 1 else 1
+    elif pane == 0:
+        if not vd.sheetstack(1): vs.pane=1
+        elif not vd.sheetstack(2) and vd.options.disp_splitwin_pct != 0: vs.pane=2
+        else: vs.pane = vd.activePane
+    else:
+        vs.pane = pane
+
     vd.sheets.insert(0, vs)
 
     if vs.precious and vs not in vd.allSheets:
@@ -1046,23 +1059,34 @@ def allSheetsSheet(vd):
 def sheetsSheet(vd):
     return SheetsSheet("sheets", source=vd.sheets)
 
+
 @VisiData.api
 def quit(vd, *sheets):
-    'Remove *sheets* from sheets stack, asking for confirmation if options.quitguard set (either global or sheet-specific).'
-    if len(vd.sheets) == len(sheets) and options.getonly('quitguard', 'global', False):
-        vd.confirm("quit last sheet? ")
+    'Remove *sheets* from sheets stack, asking for confirmation if needed.'
 
     for vs in sheets:
-        if options.getonly('quitguard', vs, False):
-            vd.draw_all()
-            vd.confirm(f'quit guarded sheet "{vs.name}?" ')
+        vs.confirmQuit('quit')
         vs.pane = 0
+        vd.remove(vs)
+
+
+@BaseSheet.api
+def confirmQuit(vs, verb='quit'):
+    if vs.options.quitguard:
+        if vs.precious and vs.hasBeenModified:
+            vd.draw_all()
+            vd.confirm(f'{verb} modified sheet "{vs.name}?" ')
+    elif vs.options.getonly('quitguard', vs, False):  # if this sheet is specifically guarded
+        vd.draw_all()
+        vd.confirm(f'{verb} guarded sheet "{vs.name}?" ')
 
 
 @BaseSheet.api
 def preloadHook(sheet):
     'Override to setup for reload().'
-    pass
+    sheet.confirmQuit('reload')
+
+    sheet.hasBeenModified = False
 
 
 @VisiData.api
@@ -1070,6 +1094,17 @@ def newSheet(vd, name, ncols, **kwargs):
     return Sheet(name, columns=[SettableColumn() for i in range(ncols)], **kwargs)
 
 
+@BaseSheet.api
+def releaseMemory(vs):
+    'Release largest memory consumer refs on *vs* to free up memory.'
+    if isinstance(vs.source, visidata.Path):
+        vs.source.lines.clear() # clear cache of read lines
+
+    if vs.precious: # only precious sheets have meaningful data
+        vs.rows.clear()
+        vd.allSheets.remove(vs)
+
+
 @Sheet.api
 def updateColNames(sheet, rows, cols, overwrite=False):
     vd.addUndoColNames(cols)
@@ -1078,12 +1113,14 @@ def updateColNames(sheet, rows, cols, overwrite=False):
             c.name = "\n".join(c.getDisplayValue(r) for r in rows)
 
 @BaseSheet.api
-def splitPane(sheet, pct):
-    vd.stackedSheets[1:] or vd.fail("need 2 sheets for splitpane")
-    if not options.disp_splitwin_pct:
-        sheet.pane=1 if sheet.pane == 2 else 2
-    options.disp_splitwin_pct = pct
+def splitPane(sheet, pct=None):
+    if vd.activeStack[1:]:
+        undersheet = vd.activeStack[1]
+        pane = 1 if undersheet.pane == 2 else 2
+        vd.push(undersheet, pane=pane)
+        vd.activePane = pane
 
+    vd.options.disp_splitwin_pct = pct
 
 IndexSheet.class_options.header = 0
 IndexSheet.class_options.skip = 0
@@ -1093,7 +1130,7 @@ BaseSheet.init('pane', lambda: 1)
 globalCommand('S', 'sheets-stack', 'vd.push(vd.sheetsSheet)', 'open Sheets Stack: join or jump between the active sheets on the current stack')
 globalCommand('gS', 'sheets-all', 'vd.push(vd.allSheetsSheet)', 'open Sheets Sheet: join or jump between all sheets from current session')
 
-BaseSheet.addCommand('^R', 'reload-sheet', 'preloadHook(); vd.addUndoReload(rows, columns); reload(); recalc(); status("reloaded")', 'reload current sheet'),
+BaseSheet.addCommand('^R', 'reload-sheet', 'preloadHook(); reload(); recalc(); status("reloaded")', 'reload current sheet'),
 Sheet.addCommand('^G', 'show-cursor', 'status(statusLine)', 'show cursor position and bounds of current sheet on status line'),
 
 Sheet.addCommand('!', 'key-col', 'toggleKeys([cursorCol])', 'toggle current column as a key column')
@@ -1126,12 +1163,14 @@ SheetsSheet.addCommand('gz^C', 'cancel-rows', 'for vs in selectedRows: cancelThr
 SheetsSheet.addCommand(ENTER, 'open-row', 'dest=cursorRow; vd.sheets.remove(sheet) if not sheet.precious else None; vd.push(openRow(dest))', 'open sheet referenced in current row')
 
 BaseSheet.addCommand('q', 'quit-sheet',  'vd.quit(sheet)', 'quit current sheet')
+BaseSheet.addCommand('Q', 'quit-sheet-free',  'releaseMemory(); vd.quit(sheet)', 'discard current sheet and free memory')
 globalCommand('gq', 'quit-all', 'vd.quit(*vd.sheets)', 'quit all sheets (clean exit)')
 
-BaseSheet.addCommand('Z', 'splitwin-half', 'splitPane(-options.disp_splitwin_pct or -50)', 'split screen in half, so that second sheet on stack is visible in a second pane')
-BaseSheet.addCommand('gZ', 'splitwin-close', 'options.disp_splitwin_pct = 0', 'close an already split screen, current pane full screens')
-BaseSheet.addCommand('^I', 'splitwin-swap', 'vd.activePane = 1 if sheet.pane == 2 else 2', 'jump to other pane')
-BaseSheet.addCommand('zZ', 'splitwin-input', 'splitPane(input("% height for split window: ", value=options.disp_splitwin_pct))', 'split screen and queries for height of second pane, second sheet on stack is visible in second pane')
+BaseSheet.addCommand('Z', 'splitwin-half', 'splitPane(vd.options.disp_splitwin_pct or 50)', 'ensure split pane is set and push under sheet onto other pane')
+BaseSheet.addCommand('gZ', 'splitwin-close', 'vd.options.disp_splitwin_pct = 0\nfor vs in vd.activeStack: vs.pane = 1', 'close split screen')
+BaseSheet.addCommand('^I', 'splitwin-swap', 'vd.activePane = 1 if sheet.pane == 2 else 2', 'jump to inactive pane')
+BaseSheet.addCommand('g^I', 'splitwin-swap-pane', 'vd.options.disp_splitwin_pct=-vd.options.disp_splitwin_pct', 'swap panes onscreen')
+BaseSheet.addCommand('zZ', 'splitwin-input', 'vd.options.disp_splitwin_pct = input("% height for split window: ", value=vd.options.disp_splitwin_pct)', 'set split pane to specific size')
 
 BaseSheet.addCommand('^L', 'redraw', 'vd.redraw(); sheet.refresh()', 'refresh screen')
 BaseSheet.addCommand(None, 'guard-sheet', 'options.set("quitguard", True, sheet); status("guarded")', 'guard current sheet from accidental quitting')
